package fr.amapj.messagingcenter.miniproxy.core;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.util.zip.GZIPInputStream;

import org.apache.commons.io.IOUtils;

/**
 * Outils pour gerer efficacement serialization et deserialisation 
 *
 */
public class BasicHttpDeserializer 
{
	private InputStream is;


	public BasicHttpDeserializer(InputStream is)
	{
		this.is = is;
	}
	
	public int readInt() throws IOException 
	{
		byte[] bs = new byte[4];
		IOUtils.readFully(is, bs);		
		return ByteBuffer.wrap(bs).getInt();
	}
	
	public long readLong() throws IOException 
	{
		byte[] bs = new byte[8];
		IOUtils.readFully(is, bs);		
		return ByteBuffer.wrap(bs).getLong();
	}
	
	public String readString() throws IOException 
	{
		return readString(-1);
	}
	
	public String readString(int maxLen) throws IOException 
	{
		int len = readInt();
		if (maxLen>0)
		{
			if (len>maxLen)
			{
				throw new IOException("maxLen = "+maxLen+" Current len="+len);
			}
		}
		byte[] bs = new byte[len];
		IOUtils.readFully(is, bs);	
		return new String(bs,"UTF-8");
	}
	
	
	
	/**
	 * Lit une chaine de caractere dans le flux avec une décompression Gzip 
	 */
	public String readGzipString() throws IOException 
	{
		int len = readInt();
		
		byte[] bs = new byte[len];
		IOUtils.readFully(is, bs);
		
		ByteArrayInputStream bais = new ByteArrayInputStream(bs);
		GZIPInputStream gis = new GZIPInputStream(bais);
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		IOUtils.copy(gis, baos);
		
		baos.close();
		gis.close();

		return new String(baos.toByteArray(),"UTF-8");      
	}
	
	
	
	
	public FileInfo readFileAndCopyTo(File directory) throws IOException 
	{
		FileInfo f = new FileInfo();
		f.fileName = readString();
		f.fileSize = readLong();
		File dest = new File(directory,f.fileName);
		FileOutputStream fos = new FileOutputStream(dest);
		
		byte[] bs = new byte[1024*128];
		IOUtils.copyLarge(is, fos, 0, f.fileSize, bs);
		
		return f;
	}
	
	static public class FileInfo
	{
		public String fileName;
		
		public long fileSize;
	}
}
