package fr.amapj.messagingcenter.miniproxy.core;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.util.zip.GZIPOutputStream;

/**
 * Outils pour gerer efficacement serialization et deserialisation 
 *
 */
public class BasicHttpSerializer 
{
	private OutputStream os;


	public BasicHttpSerializer(OutputStream os)
	{
		this.os = os;
	}
	
	public void writeInt(int a) throws IOException 
	{
		os.write(ByteBuffer.allocate(4).putInt(a).array());
	}
	
	public void writeLong(long a) throws IOException 
	{
		os.write(ByteBuffer.allocate(8).putLong(a).array());
	}
	
	public void writeString(String str) throws IOException 
	{
		byte[] bs = str.getBytes("UTF-8");
		writeInt(bs.length);
		os.write(bs);
	}
	
	/**
	 * Compresse la chaine de caractère puis l'écrit dans le flux 
	 */
	public void writeGzipString(String str) throws IOException 
	{
		// 
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		GZIPOutputStream gos = new GZIPOutputStream(baos);
		gos.write(str.getBytes("UTF-8"));
		gos.close();
		
		writeInt(baos.size());
		
		baos.writeTo(os);
	}
	

	public void writeFile(File f) throws IOException 
	{
		writeString(f.getName());
		writeLong(f.length());
		
		FileInputStream fis = new FileInputStream(f);
		
		try
		{
			byte[] bs = new byte[1024*128];
			int len= fis.read(bs);
			while(len!=-1)
			{
				os.write(bs, 0, len);
				len= fis.read(bs);
			}
		}
		finally
		{
			fis.close();
		}
	}

	public void flush() throws IOException 
	{
		os.flush();
	}
}
