package fr.amapj.messagingcenter.miniproxy.core;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

abstract public class MiniProxyServer<T extends Enum<T>> 
{
	private final static Logger logger = LogManager.getLogger();
	private String key;
	private Class<T> enumClass;
	
	protected MiniProxyServer(String key,Class<T> enumClass)
	{
		this.key = key;
		this.enumClass = enumClass;
		
	}
	
	// Should be ovveriden by server 
	abstract protected Object processServer(T method,Object request);
	
	
	// Should be ovveriden by server
	abstract protected Class<?> findRequestClass(T method);

	
	
	
	public  void processRequest(InputStream is,OutputStream os) throws IOException
	{
		logger.info("Debut processRequest");
		
		BasicHttpSerializer bos = new BasicHttpSerializer(os);
		BasicHttpDeserializer bis = new BasicHttpDeserializer(is);
		
		// Check the key
		String k = bis.readString(200);
		if (key.equals(k)==false)
		{
			logger.info("Clé invalide");
			throw new IOException("Clé invalide");
		}
		
		// 
		String methodName = bis.readString();
		
		logger.info("Traitement de la requete :"+methodName);
		
		T method = Enum.valueOf(enumClass, methodName);
		
		String in = bis.readGzipString();
		
		Class<?> requestClass = findRequestClass(method);
		
		Object request = MiniProxyGson.createGson().fromJson(in,requestClass);
		
		Object response = processServer(method,request);
				
		String str = MiniProxyGson.createGson().toJson(response);
		bos.writeGzipString(str);
			
	}
}
