/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.view.views.gestioncontratsignes.modifiermasse.paiement;

import com.vaadin.data.util.BeanItem;
import com.vaadin.server.Sizeable.Unit;
import com.vaadin.shared.ui.label.ContentMode;
import com.vaadin.ui.TextField;

import fr.amapj.model.models.contrat.modele.GestionPaiement;
import fr.amapj.model.models.contrat.modele.StrategiePaiement;
import fr.amapj.service.services.gestioncontrat.GestionContratService;
import fr.amapj.service.services.gestioncontrat.ModeleContratDTO;
import fr.amapj.service.services.mespaiements.PaiementService;
import fr.amapj.view.engine.popup.formpopup.validator.StringLengthValidator;
import fr.amapj.view.views.gestioncontrat.editorpart.GestionContratEditorPart;
import fr.amapj.view.views.gestioncontrat.editorpart.utils.GestionContratPaiementUtils;

/**
 * Permet de supprimer ou d'activer la gestion des paiements sur un contrat
 * 
 *
 */
public class PopupPaiementGererPasGerer extends GestionContratEditorPart
{
	private int nbInscrits;
	private boolean modeActivation;
	private TextField tf;	

	public enum Step
	{
		INTRO , STEP2 , STEP3;
	}

	/**
	 * 
	 */
	public PopupPaiementGererPasGerer(Long id)
	{
		setWidth(80);
		popupTitle = "Activer / Supprimer la gestion des paiements pour un contrat";
		
		// Chargement de l'objet  à modifier
		modeleContrat = new GestionContratService().loadModeleContrat(id);
		new GestionContratPaiementUtils().checkAndUpdateStrategieAndSaisiePaiementCalculDate(modeleContrat);
		
		item = new BeanItem<ModeleContratDTO>(modeleContrat);
		
		modeActivation = modeleContrat.gestionPaiement==GestionPaiement.NON_GERE;
		
		nbInscrits =  new GestionContratService().getNbInscrits(id);

	}
	
	@Override
	protected void configure()
	{
		add(Step.INTRO,()->addIntro());
		add(Step.STEP2,()->addStep2(),()->checkStep2());
		add(Step.STEP3,()->addStep3());
	}

	private void addIntro()
	{
		// Titre
		setStepTitle("les informations générales.");

		
		if (modeActivation)
		{
			String str = "Ce contrat ne gére pas les paiements pour le moment. En continuant, vous allez pouvoir activer la gestion des paiement.<br/><br/>"+
						 "Par contre, "+nbInscrits+" adhérents ont déjà souscrits à ce contrat, pour ces personnes il sera IMPERATIF de saisir manuellement les paiements.<br/>"+
						 " Cliquer sur Continuer pour activer la gestion des paiements";
			addLabel(str, ContentMode.HTML);
		}
		else
		{
			String str = "Ce contrat gére les paiements pour le moment. En continuant, vous allez SUPPRIMER toutes les informations relatives au paiement<br/><br/>"+
					 	 nbInscrits+" adhérents ont déjà souscrits à ce contrat, TOUTES LES INFORMATIONS DE PAIEMENT SERONT EFFACEES<br/>"+
					 	 " Cliquer sur Continuer pour confirmer cette suppression";
			addLabel(str, ContentMode.HTML);
		}
	}

	
	private void addStep2() 
	{
		if (modeActivation)
		{	
			// Valeur par défaut
			modeleContrat.strategiePaiement = StrategiePaiement.UN_CHEQUE_PAR_MOIS_LISSE_MODIFIABLE_1ERE_LIVRAISON;
			
			//
			drawTypePaiement(new StrategiePaiement[] { StrategiePaiement.NON_GERE});
			return ;
		}
		
		setStepTitle("confirmation");
		
		boolean b = new PaiementService().canSuppressPaiementModeleContrat(modeleContrat.id);
		if (b==false)
		{
			addLabel("Ce contrat contient des chèques à l'état AMAP ou PRODUCTEUR. Il faut supprimer d'abord tous ces chèques pour pouvoir continuer", ContentMode.HTML);
			setBackOnlyMode();
			return;
		}
		
		
		addLabel("Vous allez effacer toutes les informations de paiements. Si vous souhaitez continuer, vous devez saisir ci dessous le texte <br/>JE CONFIRME<br/> puis cliquer sur Continuer",ContentMode.HTML);
		
		tf = new TextField("");
		tf.setValue("");
		tf.setImmediate(true);
		form.addComponent(tf);
		
	}
	
	private String checkStep2() 
	{
		if (modeActivation)
		{
			return checkTypePaiement();
		}
		
		
		if (tf.getValue().equals("JE CONFIRME"))
		{
			return null;
		}
		else
		{
			return "Pour pouvoir continuer, vous devez saisir le texte JE CONFIRME"; 
		}		
	}
	
	private void addStep3() 
	{
		if (modeActivation)
		{
			drawDetailPaiement(true);
			return ;
		}		
		
		setStepTitle("les informations complémentaires sur le paiement");
		
		TextField f = (TextField) addTextField("Texte affiché dans la fenêtre paiement", "textPaiement",new StringLengthValidator(0, 2048));
		f.setMaxLength(2048);
		f.setHeight(5, Unit.CM);
		
		
		addLabel("Cliquez maintenant sur Sauvegarder et la modification sera faite",ContentMode.HTML);
	}

	
	
	

	@Override
	protected void performSauvegarder()
	{
		if (modeActivation)
		{
			new PaiementService().updateInfoPaiement(modeleContrat,true);
		}
		else
		{
			new PaiementService().deleteInfoPaiement(modeleContrat);
		}
	}

	@Override
	protected Class getEnumClass()
	{
		return Step.class;
	}
	
	
	/**
	 * 
	 */
	@Override
	protected String checkInitialCondition()
	{
		if (nbInscrits==0)
		{
			String str = "Aucun adhérent n'est inscrit à ce contrat. Vous pouvez donc modifier les règles de paiements librement, "
					+ "en allant dans \"Gestion des contrats vierges\", puis vous cliquez sur le bouton \"Modifier\" puis \"les informations de paiements\"<br/>";
			return str;
		}
		
		return null;
	}
}
