/*
 *  Copyright 2013-2050 Emmanuel BRUN (contact@amapj.fr)
 * 
 *  This file is part of AmapJ.
 *  
 *  AmapJ is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.

 *  AmapJ is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with AmapJ.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 */
 package fr.amapj.view.views.mescontrats;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import com.vaadin.server.FontAwesome;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;

import fr.amapj.service.services.mescontrats.small.SmallContratDTO;
import fr.amapj.service.services.mescontrats.small.SmallContratsService;
import fr.amapj.service.services.session.SessionManager;
import fr.amapj.view.engine.popup.PopupListener;
import fr.amapj.view.engine.template.FrontOfficeView;
import fr.amapj.view.engine.tools.BaseUiTools;


/**
 * Page permettant à l'utilisateur de gérer ses contrats
 * 
 */
public class MesContratsView extends FrontOfficeView implements  PopupListener
{
	
	static public String LABEL_RUBRIQUE = "rubrique";
	static public String LABEL_TITRECONTRAT = "titrecontrat";
	static public String PANEL_UNCONTRAT = "uncontrat";
	static public String BUTTON_PRINCIPAL = "principal";
	
	VerticalLayout layout = null;
	
	
	private List<SmallContratDTO> existingContrats;
	
	@Override
	public String getMainStyleName()
	{
		return "contrat";
	}
	
	/**
	 * 
	 */
	@Override
	public void enter()
	{	
		refresh();
	}

	
	/**
	 * Ajoute un label sur toute la largeur à la ligne indiquée
	 */
	private Label addLabel(VerticalLayout layout, String str)
	{
		Label tf = new Label(str);
		tf.addStyleName(LABEL_RUBRIQUE);
		layout.addComponent(tf);
		return tf;
		
	}

	public void refresh()
	{
		//
		List<SmallContratDTO> mesContratsDTO = new SmallContratsService().getMesContrats(SessionManager.getUserId());
		
		// Pour les nouveaux contrats, tri par date de fin des inscription puis par nom 
		// Les cartes prépayées sont en dernier  
		Comparator<SmallContratDTO> cmp = Comparator.comparing( (SmallContratDTO e)->e.dateFinInscription, Comparator.nullsLast(Comparator.naturalOrder())).thenComparing(e->e.nom);
				
		
		List<SmallContratDTO> newContrats = mesContratsDTO.stream().filter(e->e.contratId==null && e.inscriptionDTO.isRetardataire==false).sorted(cmp).collect(Collectors.toList());
		existingContrats = mesContratsDTO.stream().filter(e->e.contratId!=null).sorted(Comparator.comparing(e->e.dateFin)).collect(Collectors.toList());
		
		//
		layout = this;
		layout.removeAllComponents();
		InscriptionButtonModule inscriptionButtonModule = new InscriptionButtonModule(this);
			
		// Le titre
		addLabel(layout,"Les nouveaux contrats disponibles");
		
		
		// la liste des nouveaux contrats 
		for (SmallContratDTO c : newContrats)
		{
			Panel p = new Panel();
			p.addStyleName(PANEL_UNCONTRAT);
			
			HorizontalLayout hl = new HorizontalLayout();
			hl.setMargin(true);
			hl.setSpacing(true);
			hl.setWidth("100%");
			
			VerticalLayout vl = new VerticalLayout();
			Label lab = new Label(c.nom);
			lab.addStyleName(LABEL_TITRECONTRAT);
			vl.addComponent(lab);
			
			BaseUiTools.addHtmlLabel(vl, c.inscriptionDTO.libContrat, "libelle-contrat");
			
			
			hl.addComponent(vl);
			hl.setExpandRatio(vl, 1);
			
			inscriptionButtonModule.buildBloc(hl, c);
			
			p.setContent(hl);
			
			layout.addComponent(p);
			
		}
		
		
		// Le titre
		addLabel(layout,"Mes contrats existants");
	
		
		// la liste des contrats existants 
		for (SmallContratDTO c : existingContrats)
		{
			Panel p = new Panel();
			p.addStyleName(PANEL_UNCONTRAT);
			
			HorizontalLayout hl = new HorizontalLayout();
			hl.setMargin(true);
			hl.setSpacing(true);
			hl.setWidth("100%");
			
			VerticalLayout vl = new VerticalLayout();
			Label lab = new Label(c.nom);
			lab.addStyleName(LABEL_TITRECONTRAT);
			vl.addComponent(lab);
						
			BaseUiTools.addHtmlLabel(vl, c.inscriptionDTO.libContrat, "libelle-contrat");
			
			hl.addComponent(vl);
			hl.setExpandRatio(vl, 1);
			
			inscriptionButtonModule.buildBloc(hl, c);
			
			p.setContent(hl);
			
			layout.addComponent(p);
			
		}
		
		// Le bouton pour télécharger les contrats
		if (existingContrats.size()>0)
		{
			Button telechargerButton = new Button("Imprimer mes contrats ...");
			telechargerButton.setIcon(FontAwesome.PRINT);
			telechargerButton.addStyleName("borderless");
			telechargerButton.addStyleName("large");
			telechargerButton.addClickListener(e->handleTelecharger());
					
			layout.addComponent(telechargerButton);
			layout.setComponentAlignment(telechargerButton, Alignment.MIDDLE_LEFT);
		
		}
	}
	

	private void handleTelecharger()
	{
		new TelechargerMesContrat().displayPopupTelechargerMesContrat(existingContrats, this);
	}


	@Override
	public void onPopupClose()
	{
		refresh();
		
	}


}
